@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -31,6 +31,7 @@
 #include <dpkg/i18n.h>
 #include <dpkg/dpkg.h>
 #include <dpkg/dpkg-db.h>
+#include <dpkg/string.h>
 
 #include "dselect.h"
 #include "bindings.h"
@@ -103,7 +104,7 @@ void packagelist::discardheadings() {
     delete head;
     head= next;
   }
-  headings= 0;
+  headings = nullptr;
 }
 
 void packagelist::addheading(enum ssavailval ssavail,
@@ -126,7 +127,7 @@ void packagelist::addheading(enum ssavailval ssavail,
         section ? section : "<null>");
 
   struct pkgset *newset = new pkgset;
-  newset->name = NULL;
+  newset->name = nullptr;
   struct pkginfo *newhead = &newset->pkg;
   newhead->set = newset;
   newhead->priority= priority;
@@ -220,7 +221,7 @@ void packagelist::ensurestatsortinfo() {
             "packagelist[%p]::ensurestatsortinfos() i=%d ssavail=%d",
             this, index, table[index]->ssavail);
     }
-    calcssadone= 1;
+    calcssadone = true;
     break;
   case sso_state:
     debug(dbg_general, "packagelist[%p]::ensurestatsortinfos() calcsssdone=%d",
@@ -253,7 +254,7 @@ void packagelist::ensurestatsortinfo() {
             "packagelist[%p]::ensurestatsortinfos() i=%d ssstate=%d",
             this, index, table[index]->ssstate);
     }
-    calcsssdone= 1;
+    calcsssdone = true;
     break;
   default:
     internerr("unknown statsortorder %d", statsortorder);
@@ -271,7 +272,7 @@ void packagelist::sortmakeheads() {
         this, sortorder, statsortorder);
 
   int nrealitems= nitems;
-  addheading(ssa_none,sss_none,pkginfo::pri_unset,0,0);
+  addheading(ssa_none, sss_none, pkginfo::pri_unset, nullptr, nullptr);
 
   assert(sortorder != so_unsorted);
   if (sortorder == so_alpha && statsortorder == sso_unsorted) { sortinplace(); return; }
@@ -281,7 +282,7 @@ void packagelist::sortmakeheads() {
 
   struct pkginfo *lastpkg;
   struct pkginfo *thispkg;
-  lastpkg= 0;
+  lastpkg = nullptr;
   int a;
   for (a=0; a<nrealitems; a++) {
     thispkg= table[a]->pkg;
@@ -327,15 +328,16 @@ void packagelist::sortmakeheads() {
 
     if (ssdiff)
       addheading(ssavail,ssstate,
-                 pkginfo::pri_unset,0, 0);
+                 pkginfo::pri_unset, nullptr, nullptr);
 
     if (sortorder == so_section && sectiondiff)
       addheading(ssavail,ssstate,
-                 pkginfo::pri_unset,0, thispkg->section ? thispkg->section : "");
+                 pkginfo::pri_unset, nullptr,
+                 thispkg->section ? thispkg->section : "");
 
     if (sortorder == so_priority && prioritydiff)
       addheading(ssavail,ssstate,
-                 thispkg->priority,thispkg->otherpriority, 0);
+                 thispkg->priority, thispkg->otherpriority, nullptr);
 
     if (sortorder != so_alpha && (prioritydiff || sectiondiff))
       addheading(ssavail,ssstate,
@@ -361,13 +363,13 @@ void packagelist::initialsetup() {
   nallocated= allpackages+150; // will realloc if necessary, so 150 not critical
   table= new struct perpackagestate*[nallocated];
 
-  depsdone= 0;
-  unavdone= 0;
-  currentinfo= 0;
-  headings= 0;
-  verbose= 0;
-  calcssadone= calcsssdone= 0;
-  searchdescr= 0;
+  depsdone = nullptr;
+  unavdone = nullptr;
+  currentinfo = nullptr;
+  headings = nullptr;
+  verbose = false;
+  calcssadone = calcsssdone = false;
+  searchdescr = false;
 }
 
 void packagelist::finalsetup() {
@@ -392,7 +394,8 @@ packagelist::packagelist(keybindings *kb) : baselist(kb) {
     if (pkg->status == pkginfo::stat_notinstalled &&
         !pkg->files &&
         pkg->want != pkginfo::want_install) {
-      pkg->clientdata= 0; continue;
+      pkg->clientdata = nullptr;
+      continue;
     }
     // treat all unknown packages as already seen
     state->direct= state->original= (pkg->want == pkginfo::want_unknown ? pkginfo::want_purge : pkg->want);
@@ -409,7 +412,7 @@ packagelist::packagelist(keybindings *kb) : baselist(kb) {
     }
     state->dpriority= dp_must;
     state->selected= state->suggested;
-    state->uprec= 0;
+    state->uprec = nullptr;
     state->relations.init();
     pkg->clientdata= state;
     table[nitems]= state;
@@ -419,7 +422,7 @@ packagelist::packagelist(keybindings *kb) : baselist(kb) {
 
   if (!nitems)
     ohshit(_("there are no packages"));
-  recursive= 0;
+  recursive = false;
   sortorder= so_priority;
   statsortorder= sso_avail;
   versiondisplayopt= vdo_both;
@@ -431,7 +434,7 @@ packagelist::packagelist(keybindings *kb, pkginfo **pkgltab) : baselist(kb) {
   // takes over responsibility for pkgltab (recursive)
   initialsetup();
 
-  recursive= 1;
+  recursive = true;
   nitems= 0;
   if (pkgltab) {
     add(pkgltab);
@@ -444,7 +447,9 @@ packagelist::packagelist(keybindings *kb, pkginfo **pkgltab) : baselist(kb) {
   finalsetup();
 }
 
-void perpackagestate::free(int recursive) {
+void
+perpackagestate::free(bool recursive)
+{
   if (pkg->set->name) {
     if (modstatdb_get_status() == msdbrw_write) {
       if (uprec) {
@@ -457,7 +462,7 @@ void perpackagestate::free(int recursive) {
             !(pkg->want == pkginfo::want_unknown && selected == pkginfo::want_purge)) {
           pkg->want= selected;
         }
-        pkg->clientdata= 0;
+        pkg->clientdata = nullptr;
       }
     }
     relations.destroy();
@@ -492,10 +497,10 @@ packagelist::checksearch(char *rx)
 {
   int r,opt = REG_NOSUB;
 
-  if (!rx || !*rx)
+  if (str_is_unset(rx))
     return false;
 
-  searchdescr=0;
+  searchdescr = false;
   if (searchstring[0]) {
     regfree(&searchfsm);
     searchstring[0]=0;
@@ -517,12 +522,13 @@ packagelist::checksearch(char *rx)
      if (rx[r]=='i')
        opt|=REG_ICASE;
      else if (rx[r]=='d')
-       searchdescr=1;
+       searchdescr = true;
      r++;
    }
   }
 
-  if ((r=regcomp(&searchfsm, rx, opt))!=0) {
+  r = regcomp(&searchfsm, rx, opt);
+  if (r != 0) {
     displayerror(_("error in regular expression"));
     return false;
   }
@@ -539,15 +545,15 @@ packagelist::matchsearch(int index)
   if (!name)
     return false;	/* Skip things without a name (seperators) */
 
-  if (regexec(&searchfsm, name, 0, NULL, 0) == 0)
+  if (regexec(&searchfsm, name, 0, nullptr, 0) == 0)
     return true;
 
   if (searchdescr) {
     const char *descr = table[index]->pkg->available.description;
-    if (!descr || !*descr)
+    if (str_is_unset(descr))
       return false;
 
-    if (regexec(&searchfsm, descr, 0, NULL, 0)==0)
+    if (regexec(&searchfsm, descr, 0, nullptr, 0) == 0)
       return true;
   }
 
@@ -576,12 +582,12 @@ pkginfo **packagelist::display() {
     if (doupdate() == ERR)
       ohshite(_("doupdate failed"));
     signallist= this;
-    if (sigprocmask(SIG_UNBLOCK, &sigwinchset, 0))
+    if (sigprocmask(SIG_UNBLOCK, &sigwinchset, nullptr))
       ohshite(_("failed to unblock SIGWINCH"));
     do
     response= getch();
     while (response == ERR && errno == EINTR);
-    if (sigprocmask(SIG_BLOCK, &sigwinchset, 0))
+    if (sigprocmask(SIG_BLOCK, &sigwinchset, nullptr))
       ohshite(_("failed to re-block SIGWINCH"));
     if (response == ERR)
       ohshite(_("getch failed"));
@@ -598,26 +604,23 @@ pkginfo **packagelist::display() {
   if (interp->qa == qa_quitnochecksave ||
       modstatdb_get_status() == msdbrw_readonly) {
     debug(dbg_general, "packagelist[%p]::display() done - quitNOcheck", this);
-    return 0;
+    return nullptr;
   }
 
   if (recursive) {
     retl= new pkginfo*[nitems+1];
     for (index=0; index<nitems; index++) retl[index]= table[index]->pkg;
-    retl[nitems]= 0;
+    retl[nitems] = nullptr;
     debug(dbg_general, "packagelist[%p]::display() done, retl=%p", this, retl);
     return retl;
   } else {
-    packagelist *sub= new packagelist(bindings,0);
+    packagelist *sub = new packagelist(bindings, nullptr);
     for (index=0; index < nitems; index++)
       if (table[index]->pkg->set->name)
         sub->add(table[index]->pkg);
     repeatedlydisplay(sub,dp_must);
     debug(dbg_general,
           "packagelist[%p]::display() done, not recursive no retl", this);
-    return 0;
+    return nullptr;
   }
 }
-
-/* vi: sw=2 ts=8
- */
