@@ -3,7 +3,7 @@
  * parse.c - database file parsing, main package/field loop
  *
  * Copyright © 1995 Ian Jackson <ian@chiark.greenend.org.uk>
- * Copyright © 2006,2008-2012 Guillem Jover <guillem@debian.org>
+ * Copyright © 2006,2008-2013 Guillem Jover <guillem@debian.org>
  *
  * This is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program.  If not, see <http://www.gnu.org/licenses/>.
+ * along with this program.  If not, see <https://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -229,7 +229,7 @@ pkg_parse_verify(struct parsedb_state *ps,
     if (pkg->configversion.version) {
       if (pkg->status == stat_installed || pkg->status == stat_notinstalled)
         parse_error(ps,
-                    _("Configured-Version for package with inappropriate Status"));
+                    _("Config-Version for package with inappropriate Status"));
     } else {
       if (pkg->status == stat_installed)
         pkg->configversion = pkgbin->version;
@@ -490,12 +490,12 @@ static enum parsedbtype
 parse_get_type(struct parsedb_state *ps, enum parsedbflags flags)
 {
   if (flags & pdb_recordavailable) {
-    if (flags & pdb_deb_control)
+    if (flags & pdb_single_stanza)
       return pdb_file_control;
     else
       return pdb_file_available;
   } else {
-    if (flags & pdb_deb_control)
+    if (flags & pdb_single_stanza)
       return pdb_file_update;
     else
       return pdb_file_status;
@@ -582,6 +582,11 @@ parse_stanza(struct parsedb_state *ps, struct field_state *fs,
     while (!parse_EOF(ps) && !isspace(c) && c != ':' && c != MSDOS_EOF_CHAR)
       c = parse_getc(ps);
     fs->fieldlen = ps->dataptr - fs->fieldstart - 1;
+    if (fs->fieldlen == 0)
+      parse_error(ps,  _("empty field name"));
+    if (fs->fieldstart[0] == '-')
+      parse_error(ps,  _("field name '%.*s' cannot start with hyphen"),
+                  fs->fieldlen, fs->fieldstart);
 
     /* Skip spaces before ‘:’. */
     while (!parse_EOF(ps) && c != '\n' && isspace(c))
@@ -639,7 +644,6 @@ parse_stanza(struct parsedb_state *ps, struct field_state *fs,
           break;
 
         parse_ungetc(c, ps);
-        c = '\n';
         blank_line = true;
       } else if (blank_line && !isspace(c)) {
         blank_line = false;
